
#ifndef __C_PROGRAM_H
#define __C_PROGRAM_H


#include "render/Canvas.h"
#include "render/Drawing.h"
#include "fx/ImageEffect.h"
#include "flake/build/Builder.h"
#include "flake/SpikeSystem.h"


class CTextEffect;
class CTimeChecker;
class CMemoryChecker;



struct SParams {
	SParams()
		: mAllowedTime(-1), mAllowedMemory(-1), mCPUSpeed(-1),
		mPreEffect(0), mPostEffect(0),
		mSystemBuilder(0),
		mRandomSeed(0), mIterations(-1) { }

	// contest required

	int		mAllowedTime;	// secs
	int		mAllowedMemory;	// mb
	int		mCPUSpeed;		// mhz

	// look&feel

	IImageEffect*	mPreEffect;
	IImageEffect*	mPostEffect;
	IBuilder*		mSystemBuilder;
	int		mRandomSeed;
	int		mIterations;
	bool	mDark;
};



class CProgram {
public:
	enum { IMG_WIDTH = 800 };
	enum { IMG_HEIGHT = 600 };
	enum { FONT_WIDTH = 14 };

public:
	CProgram();
	~CProgram();

	/** @return true on success, false if execution should stop. */
	bool processParams( int argc, char** argv );

	void createCanvases();

	void run();

	void finish();

private:
	void printHelp() const;	
	void restrictedRun( CSpikeSystem& ls );
	
private:
	CCanvas*	mColorCanvas;
	CCanvas*	mDepthCanvas;
	CDrawing*	mDrawing;

	CCanvas*	mTextCanvas;
	CTextEffect*	mTextEffect;

	CTimeChecker*	mTimeChecker;
	CMemoryChecker*	mMemoryChecker;

	SParams		mParams;
};


#endif
