
#ifndef __BITMAP_SERIALIZER_H
#define __BITMAP_SERIALIZER_H

#include <cstdio>
#include <cassert>

#include "../render/Canvas.h"


// -----------------------------------------------------------------------

/**
 *  BMP format builder.
 */
class CBuilderBMP {
public:
	static void build( FILE* file, const CCanvas& canvas );
	static CCanvas* reverseBuild( FILE* file );
};


// -----------------------------------------------------------------------

/**
 *  Bitmap serializer. Saves bitmap to a file; using given format builder.
 *  @param BUILDER Builder to handle file format.
 */
template<class BUILDER>
class CBitmapSerializer {
public:
	static void serialize( const char* fileName, const CCanvas& canvas );
	static CCanvas* deserialize( const char* fileName );
};

template<class BUILDER>
void CBitmapSerializer<BUILDER>::serialize( const char* fileName, const CCanvas& canvas )
{
	FILE *file = fopen( fileName, "wb" );
	if( !file )
		return;
	BUILDER::build( file, canvas );
	fclose( file );
};

template<class BUILDER>
CCanvas* CBitmapSerializer<BUILDER>::deserialize( const char* fileName )
{
	FILE *file = fopen( fileName, "rb" );
	if( !file )
		return 0;
	CCanvas* cnv = BUILDER::reverseBuild( file );
	fclose( file );
	return cnv;
};


#endif
