
#ifndef __SIMPLE_EFFECTS_H
#define __SIMPLE_EFFECTS_H


#include "ImageEffect.h"
#include "../util/Random.h"
#include <string>



class CUniformFillEffect : public IImageEffect {
public:
	CUniformFillEffect( CCanvas::pixel color )
		: mColor( color ) { }
	virtual void perform( CDrawing& draw, CCanvas& color, CCanvas& depth ) {
		draw.fill( color, mColor );
		draw.fillFP( depth, 1.0e6f );
	}
private:
	CCanvas::pixel	mColor;
};


class CBlurEffect : public IImageEffect {
public:
	CBlurEffect( int size )
		: mSize( size ) { }

	virtual void perform( CDrawing& draw, CCanvas& color, CCanvas& depth );
	
private:
	int	mSize;
};



class CBackgroundEffect : public IImageEffect {
public:
	CBackgroundEffect( int seed, bool dark )
		: mRandom(seed), mDark(dark) { }
	
	virtual void perform( CDrawing& draw, CCanvas& color, CCanvas& depth );
	
private:
	CRandom	mRandom;
	bool mDark;
};


class CBackgroundEffect2 : public IImageEffect {
public:
	CBackgroundEffect2( int seed, bool dark )
		: mRandom(seed), mDark(dark) { }
	
	virtual void perform( CDrawing& draw, CCanvas& color, CCanvas& depth );
	
private:
	CRandom	mRandom;
	bool mDark;
};


class CTextEffect : public IImageEffect {
public:
	CTextEffect( const CCanvas& image, int size );

	static void initialize();
	
	void setText( const char* text, int x, int y, bool sub ) { mText = text; mX=x; mY=y; mSub = sub; }
	
	virtual void perform( CDrawing& draw, CCanvas& color, CCanvas& depth );
	
private:
	const CCanvas&	mImage;
	int				mSize;

	std::string		mText;
	int				mX;
	int				mY;
	bool			mSub;

	static const char* mLetters;
	static int*		mIndexByChar;
};


#endif
