
#ifndef __VEKTORIUS_H__
#define __VEKTORIUS_H__

#include <cmath>


#pragma pack(1)

struct SVector {
public:
	SVector();
	SVector( float x, float y, float z );
	SVector( const SVector& v );
	
	void set( float x, float y, float z ) { this->x = x; this->y = y; this->z = z; };

	SVector operator+ ( const SVector &a ) const;
	SVector operator- ( const SVector &a ) const;
	SVector operator- () const;
	SVector operator* ( float a ) const;
	SVector operator/ ( float a ) const;
	SVector& operator= ( const SVector &a );
	SVector& operator+= ( const SVector &a );
	SVector& operator-= ( const SVector &a );
	SVector& operator*= ( float a );
	SVector& operator/= ( float a );
	
	float length() const;
	float squaredLength() const;
	void normalize();

	/** THIS.V */
	float dotProduct( const SVector &v ) const;
	/** THISxV */
	SVector crossProduct( const SVector &v ) const;

	/** Rotates about x, y, z axes (in radians). */
	void rotate( float rx, float ry, float rz );
	
public:
	float x, y, z;
};

#pragma pack()


inline SVector::SVector()
	{ }
inline SVector::SVector( float xx, float yy, float zz )
	: x(xx), y(yy), z(zz) { }
inline SVector::SVector( const SVector& v )
	: x(v.x), y(v.y), z(v.z) { }

inline bool operator== ( const SVector &a, const SVector &b )
	{ return (a.x==b.x && a.y==b.y && a.z==b.z); };
inline bool operator!= ( const SVector &a, const SVector &b )
	{ return (a.x!=b.x || a.y!=b.y || a.z!=b.z); };


inline SVector SVector::operator+ ( const SVector &a ) const
	{ return SVector(x+a.x, y+a.y, z+a.z); };
inline SVector SVector::operator- ( const SVector &a ) const
	{ return SVector(x-a.x, y-a.y, z-a.z); };
inline SVector SVector::operator- () const
	{ return SVector(-x, -y, -z); };
inline SVector SVector::operator* ( float a ) const
	{ return SVector(x*a, y*a, z*a); };
inline SVector SVector::operator/ ( float a ) const
	{ return SVector(x/a, y/a, z/a); };

inline SVector& SVector::operator= ( const SVector &a )
	{ x=a.x; y=a.y; z=a.z; return *this; };
inline SVector& SVector::operator+= ( const SVector &a )
	{ x+=a.x; y+=a.y; z+=a.z; return *this; };
inline SVector& SVector::operator-= ( const SVector &a )
	{ x-=a.x; y-=a.y; z-=a.z; return *this; };
inline SVector& SVector::operator*= ( float a )
	{ x*=a; y*=a; z*=a; return *this; };
inline SVector& SVector::operator/= ( float a )
	{ float aa = 1.0f / a; x*=aa; y*=aa; z*=aa; return *this; };

inline float SVector::length() const
	{ return sqrtf( x*x + y*y + z*z ); };
inline float SVector::squaredLength() const
	{ return x*x + y*y + z*z; };
inline void SVector::normalize()
	{ if (x==0.0 && y==0.0 && z==0.0) return; *this /= length(); }
inline float SVector::dotProduct( const SVector &v ) const
	{ return x*v.x + y*v.y + z*v.z; };


#endif


